/*************************************************************

  QR2Wiegand26.ino
  
  Translates a specific format QR code which is 
  scanned via a WaveShare Barcode Scanner Module
  with a UART connection to the Arduino UNO.

  Upon translation, the number is bit banged as
  26 bit Wiegand to the wD0/wD1 pins as defined.

  Written by Daniel Raines Communications on 27/04/2019
  http://DanielRaines.com | http://BreeCS.com
 
*************************************************************/

#include <SoftwareSerial.h>
#define BUFF_MAXLEN 16  //Buffer for data: DRC:5:65536# << SEE NOTES
#define bcRX 2          //Barcode Receive
#define bcTX 3          //Barcode Transmit
#define wD0 8           //Wiegand D0
#define wD1 9           //Wiegand D1
char barcodeBuffer[BUFF_MAXLEN];
SoftwareSerial serialData =  SoftwareSerial(bcRX, bcTX);

void setup() {
  pinMode(wD0, OUTPUT);
  pinMode(wD1, OUTPUT);
  digitalWrite(wD0, HIGH);
  digitalWrite(wD1, HIGH);
  clearBarcodeBuffer();
  serialData.begin(9600);
  //Serial.begin(9600);
}

void writeD0() {
  digitalWrite(wD0, LOW);
  delayMicroseconds(50);
  digitalWrite(wD0, HIGH);
}
void writeD1() {
  digitalWrite(wD1, LOW);
  delayMicroseconds(50);
  digitalWrite(wD1, HIGH);
}

void clearBarcodeBuffer() {
  for (int i = 0; i < BUFF_MAXLEN; i++) {
    barcodeBuffer[i] = (char)0;
  }
}

void sendWiegand(long unsigned int tokenNumber) {
  long unsigned int bitMask = 8388608;
  int LeadingParity;
  int TrailingParity;
  for (int i = 0; i < 12; i++) {
    if ((bitMask & tokenNumber) == 0) {
      LeadingParity += 0;
    } else {
      LeadingParity += 1;
    }
    bitMask = bitMask >> 1;
  }
  if (LeadingParity % 2 == 0) {
    writeD0();
  } else {
    writeD1();
  }
  delay(2);
  bitMask = 8388608;
  for (int i = 0; i < 12; i++) {
    if ((bitMask & tokenNumber) == 0) {
      writeD0();
    } else {
      writeD1();
    }
    bitMask = bitMask >> 1;
    delay(2);
  }
  for (int i = 12; i < 24; i++) {
    if ((bitMask & tokenNumber) == 0) {
      TrailingParity += 0;
      writeD0();
    } else {
      TrailingParity += 1;
      writeD1();
    }
    bitMask = bitMask >> 1;
    delay(2);
  }
  if (TrailingParity % 2 == 1) {
    writeD0();
  } else {
    writeD1();
  }
  delay(2);
}

void loop() {
  while (serialData.available() > 0) {
    clearBarcodeBuffer();
    serialData.readBytesUntil('#', barcodeBuffer, BUFF_MAXLEN);
    //Serial.println(barcodeBuffer);
    long unsigned int tokenNumber = 0;
    int tnLen = 0;
    String tnStr = "";
    if (barcodeBuffer[0] == 'D'
        && barcodeBuffer[1] == 'R'
        && barcodeBuffer[2] == 'C'
        && barcodeBuffer[3] == ':'
        && barcodeBuffer[5] == ':') {
      tnLen = String(barcodeBuffer[4]).toInt();
      if (tnLen > 0 && tnLen < 9) {
        for (int i = 0; i < tnLen; i++) {
          tnStr += barcodeBuffer[i + 6];
          tnStr.trim();
        }
        if (tnStr.length() == tnLen && tnStr != "") {
          tokenNumber = tnStr.toInt();
          if (tokenNumber > 0) {
            sendWiegand(tokenNumber);
            //Serial.println(tokenNumber);
          }
        }
      }
    }
  }
  delay(50);
}

